//
// Demo program for DJ.lib
//
//
// Written by Andrew Murray, Ratsoft (c) 1995
//
// Compile with /W/N/A/M/V
//
//
//


// Include this define in your application
#include "dj.ch"

// Some more
#define TRUE  .T.
#define FALSE .F.
#define ESC   chr(27)
#define CRLF  chr(13)+chr(10)

// For those who want to use Imperial units in modifying this code
// comment out the following define
#define METRIC

//
// Startup function
//
function main( cPaperType )

   local nLen      := NIL ,;
         nRow      := NIL ,;
         nRow2     := NIL ,;
         nRowH     := NIL ,;
         nInc      := NIL ,;
         nCol      := NIL ,;
         nRows     := NIL ,;
         nWid      := NIL ,;
         nH        := NIL ,;
         nW        := NIL ,;
         nTmp      := NIL ,;
         cTmp      := NIL ,;
         i         := NIL ,;
         cText     := NIL ,;
         aMargins  := NIL ,;
         nTMargin  := NIL ,;
         nLMargin  := NIL ,;
         nRMargin  := NIL ,;
         nBMargin  := NIL ,;
         nHCentre  := NIL ,;
         nVCentre  := NIL ,;
         cPageSize := NIL


   // Check paramter
   if ( cPaperType == NIL )
      cPageSize := "A4"
      Message( "No parameter passed...defaulting to A4 paper")
      Message( "Usage: DEMO [ A|L ] for A4 or Letter paper")
      Message( "")
   else
      cPaperType := upper(cPaperType)
      do case
         case ( cPaperType == "A" )
            cPageSize := "A4"
         case ( cPaperType == "L" )
            cPageSize := "LETTER"
         otherwise
            Message( "Usage: DEMO [ A|L ] for A4 or LETTER paper")
            Message( "")
            Quit
      endcase
   endif

   // Initialise
   Message( "Initialising")
   set(_SET_DEVICE, "PRINTER") // Send output to "printer" not screen

   // Initialise deskjet with paper sizes, orientation, units
   pr_Set( TRUE )              // Turn on sending to printer
   dj_Reset()                  // Issue a reset
   dj_Orient( "P" )
   #ifdef METRIC
      dj_Units( "M" )
   #else
      dj_Units( "I" )
   #endif
   dj_Port("LPT1")
   dj_Symbol( "PC 850" )
   dj_Quality( FALSE )         // Set print quality to draft to save ink
   dj_PageSize( cPageSize )

   // Initialise printer itself
   dj_Init()

   // Initialise some variables
   aMargins := dj_Borders()
   nTMargin := aMargins[1]
   nLMargin := aMargins[2]
   nBMargin := aMargins[3]
   nRMargin := aMargins[4]
   nHCentre := nLMargin+((nRMargin-nLMargin)/2)
   nVCentre := nTMargin+((nBMargin-nTMargin)/2)

   //
   // Write centred headings
   //
   Message( "Generating page...")
   cText := "DJ.lib A Clipper library for HP DeskJet Printers (c) Andrew Murray 1995"
   #ifdef METRIC
      nRow  := 20
   #else
      nRow  := 0.79
   #endif
   dj_SetFont( "UNIVERS__14" )
   dj_Say( nRow, nHCentre, cText, "C" )

   cText := "Ratsoft    Tel: +49 89 2711262    Fax: +49 89 2718204    CIS: 100016,3406"
   nRow  += dj_CharHt()                       // Increment row by height of current font
   dj_SetFont( "UNIVERS__10" )
   dj_Say( nRow, nHCentre, cText, "C" )

   // Draw a horizontal line between the margins
   #ifdef METRIC
      nRow  := 20
   #else
      nRow  := 0.79
   #endif
   dj_LineH( nTMargin+nRow, nLMargin, nRMargin-nLMargin)

   // Debugging ...
   dj_SetFont( "UNIVERS__6" )
   #ifdef METRIC
      nRow := 13
      nCol := 11
      cTmp := "Metric"
   #else
      nRow := 0.51
      nCol := 0.43
      cTmp := "Imperial"
   #endif
   dj_Say( nRow, nCol, cTmp, "L" )

   //
   // A selection of available fonts
   //
   #ifdef METRIC
      nRow  := 40
      nInc  := 5
   #else
      nRow  := 1.57
      nInc  := 0.20
   #endif
   Heading( nRow, "// A selection of proportional fonts supported, CGTIMES, UNIVERS in normal and bold..." )
   cText := "And therefore never send to know..."
   nRow  += nInc
   dj_SetFont( "CGTIMES__6" )
   dj_Say( nRow, nLMargin+nInc , cText, "L" )

   nRow  += nInc
   dj_SetFont( "CGTIMES__8" )
   dj_Say( nRow, nLMargin+nInc , cText, "L" )

   nRow  += nInc
   dj_SetFont( "CGTIMES__10" )
   dj_Say( nRow, nLMargin+nInc , cText, "L" )

   nRow  += nInc
   dj_SetFont( "CGTIMES__12" )
   dj_Say( nRow, nLMargin+nInc , cText, "L" )

   nRow  += nInc
   dj_SetFont( "CGTIMES__14" )
   dj_Say( nRow, nLMargin+nInc , cText, "L" )

   nRow  += nInc
   dj_SetFont( "CGTIMES__14_B" )
   dj_Say( nRow, nLMargin+nInc , cText, "L" )

   nRow  += nInc
   dj_SetFont( "UNIVERS__10" )
   dj_Say( nRow, nLMargin+nInc , cText, "L" )

   nRow  += nInc
   dj_SetFont( "UNIVERS__14" )
   dj_Say( nRow, nLMargin+nInc , cText, "L" )


   //
   // Demonstrate absolute box placement by exactly enclosing text strings
   //
   #ifdef METRIC
      nRowH := 90
      nRow  := 100
      nH    := pt2mm(12)    // Convert point height to mm
   #else
      nRowH := 3.54
      nRow  := 3.94
      nH    := pt2inch(12)
   #endif
   Heading( nRowH, "// Justify proportional fonts..." )
   cText := "CGTIMES 12 left justified"
   dj_SetFont( "CGTIMES__12" )
   dj_Say( nRow, nLMargin, cText, "L" )
   nLen := dj_StrLen(cText)
   dj_Box( nRow-nH, nLMargin, nRow, nLMargin+nLen )

   cText := "Bold and right justified"
   dj_SetFont( "CGTIMES__12_B" )
   dj_Say( nRow, nRMargin, cText, "R" )
   nLen := dj_StrLen(cText)
   dj_Box( nRow-nH, nRMargin-nLen, nRow, nRMargin )

   cText := "COURIER string centred"
   dj_SetFont( "COURIER_10_12_BI" )
   dj_Say( nRow, nHCentre, cText, "C" )
   nLen := dj_StrLen(cText)
   dj_Box( nRow-nH, nHCentre-(nLen/2), nRow, nHCentre+(nLen/2) )


   //
   // A box around the margins
   //
   #ifdef METRIC
      nRow  := 115
   #else
      nRow  := 4.53
   #endif
   Heading( nRow, "// Your DeskJet is now a page printer - draw elements such as this page's border in any order >>")
   dj_Box( nTMargin, nLMargin, nBMargin, nRMargin )


   //
   // Draw a dotted-line boxes and lines
   //
   #ifdef METRIC
      nRowH  := 125
      nInc   := 10    // Spacing
      nRow   := 130   // Top row of outer box
      nRow2  := 170   // Bottom row of outer box
   #else
      nRowH  := 4.92
      nInc   := 0.39
      nRow   := 5.12
      nRow2  := 6.69
   #endif
   Heading( nRowH, "// Draw vertical and horizontal, dashed and dotted lines at absolute page positions..." )

   dj_LineV( nRow, nHCentre, nRow2-nRow )
   dj_LineH( nRow+((nRow2-nRow)/2), nLMargin+nInc, (nRMargin-nLMargin)-(2*nInc) )

   dj_Box( nRow, nLMargin+nInc, nRow2, nRMargin-nInc, NIL, DJ_LINE_DASHED )
   dj_Box( nRow+nInc, nLMargin+(2*nInc), nRow2-nInc, nRMargin-(2*nInc), NIL, DJ_LINE_DOTTED )


   //
   // A word-wrapped column of text
   //
   #ifdef METRIC
      nRowH := 180
      nRow  := 190
      nWid  := 85
      nH    := pt2mm(12)
      nInc  := 5
   #else
      nRowH := 7.09
      nRow  := 7.48
      nWid  := 3.35
      nH    := pt2inch(12)
      nInc  := 0.20
   #endif

   Heading( nRowH, "// Handle text formatting with DJ's MEMOLINE replacements for proportional fonts...")

   // Create a long string which also contains CRLF
   cText := ;
   "No man is an island, entire of itself; "                           +;
   "every man is a piece of the continent, a part of the main; "       +;
   "if a clod be washed away by the sea, Europe is the less, "         +;
   "as well as if a promontory were, "                                 +;
   "as well as if a manor of thy friends or of thine own were; "       +;
   "any man's death diminishes me, because I am involved in mankind; " +;
   "and therefore never send to know for whom the bell tolls; "        +;
   "it tolls for thee. "                                               +;
   CRLF                                                                +;
   "John Donne, c.1572-1631, Dean of St. Paul's from 1621."

   // Set up for printing
   nCol  := nLMargin+nInc
   dj_SetFont( "CGTIMES__12" )

   // See how many rows this string is
   nRows := dj_MLCount( cText, nWid)
   dj_Box( nRow-nH, nCol, nRow+((nRows-1)*nH), nCol+nWid )

   // Print each row
   for i := 1 to nRows
      cTmp := dj_MemoLine( i )
      dj_Say( nRow, nCol, cTmp, "L" )
      nRow += nH
   next

   // Print a code fragment showing how easily DJ.lib handles memos
   #ifdef METRIC
      nRow  := 200
      nWid  := 95
   #else
      nRow  := 7.87
      nWid  := 3.74
   #endif

   nCol  := nLMargin+nWid
   dj_SetFont( "COURIER_10_12" )
   dj_Say( nRow, nCol, "// Code fragment for memo handling...")  ; nRow += nH
   dj_Say( nRow, nCol, "cText := 'No man is an island....'")     ; nRow += nH
   dj_Say( nRow, nCol, "nRows := dj_MLCount( cText, nWid )")     ; nRow += nH
   dj_Say( nRow, nCol, "for i := 1 to nRows")                    ; nRow += nH
   dj_Say( nRow, nCol, " dj_Say( nRow, nCol, dj_MemoLine(i))")   ; nRow += nH
   dj_Say( nRow, nCol, " nRow += nH")                            ; nRow += nH
   dj_Say( nRow, nCol, "next")                                   ; nRow += nH

   //
   // Now a graphic
   //
   #ifdef METRIC
      nRowH := 240
      nRow  := 242
      nW    := 20
      nH    := 13
      nInc  := 10
   #else
      nRowH :=  9.45
      nRow  :=  9.52
      nW    :=  0.78
      nH    :=  0.51
      nInc  :=  0.39
   #endif

   Heading( nRowH, "// Place PCL file anywhere on the page...")
   dj_PCL( nRow, nHCentre-(nW/2) , "ratsoft.pcl", nW, nH )

   //
   // And now send contents of DeskJet page buffer
   //
   Message( "Sending page to printer...")
   dj_Print()
   dj_Eject()
   dj_Reset()


   // Finished
   pr_Set( FALSE )
   Message( "Finished!")
   Message( "")

return( NIL )


//
// Print out heading lines
//
static function Heading( nRow, cText )

   local nCol := NIL

   dj_SetFont( "CGTIMES__12_B" )
   #ifdef METRIC
      nCol := 15
   #else
      nCol := 0.59
   #endif

   dj_Say( nRow, nCol, cText, "L" )
return( NIL )


//
// Send a messge to screen
//
static function Message( cText )

   local cOldDev := set(_SET_DEVICE, "SCREEN") ,;
         lOldSet := pr_Set( FALSE )

   Qout( cText )
   pr_Set( lOldSet )
   set(_SET_DEVICE, cOldDev )

return( NIL )
