//
// Include this define in your application
//
#include "dj.ch"

// Some more
#define TRUE  .T.
#define FALSE .F.
#define ESC   chr(27)
#define CRLF  chr(13)+chr(10)


//
// Startup function
//
function main( cPaperType )

   local nLen      := NIL ,;
         nRow      := NIL ,;
         nRow2     := NIL ,;
         nRowH     := NIL ,;
         nInc      := NIL ,;
         nCol      := NIL ,;
         nRows     := NIL ,;
         nWid      := NIL ,;
         nH        := NIL ,;
         nW        := NIL ,;
         nTmp      := NIL ,;
         cTmp      := NIL ,;
         cFont     := NIL ,;
         i         := NIL ,;
         cText     := NIL ,;
         aMargins  := NIL ,;
         nTMargin  := NIL ,;
         nLMargin  := NIL ,;
         nRMargin  := NIL ,;
         nBMargin  := NIL ,;
         nHCentre  := NIL ,;
         nVCentre  := NIL ,;
         cPageSize := NIL


   // Check paramter
   if ( cPaperType == NIL )
      cPageSize := "A4"
      Message( "No parameter passed...defaulting to A4 paper")
      Message( "Usage: DEMO [ A|L ] for A4 or Letter paper")
      Message( "")
   else
      cPaperType := upper(cPaperType)
      do case
         case ( cPaperType == "A" )
            cPageSize := "A4"
         case ( cPaperType == "L" )
            cPageSize := "LETTER"
         otherwise
            Message( "Usage: DEMO [ A|L ] for A4 or LETTER paper")
            Message( "")
            Quit
      endcase
   endif

   // Initialise
   //Message( "Initialising")
   set(_SET_DEVICE, "PRINTER") // Send output to "printer" not screen

   // Initialise deskjet with paper sizes, orientation, units
   pr_Set( TRUE )              // Turn on sending to printer
   dj_Reset()                  // Issue a reset
   dj_Orient( "P" )
   dj_Units( "M" )
   dj_Port("LPT1")
   dj_Symbol( "PC 850" )
   dj_Quality( FALSE )         // Set print quality to draft to save ink
   dj_PageSize( cPageSize )

   // Initialise printer itself
   dj_Init()

   // Initialise some variables
   aMargins := dj_Borders()
   nTMargin := aMargins[1]
   nLMargin := aMargins[2]
   nBMargin := aMargins[3]
   nRMargin := aMargins[4]
   nHCentre := nLMargin+((nRMargin-nLMargin)/2)
   nVCentre := nTMargin+((nBMargin-nTMargin)/2)

   //
   // Write centred headings
   //
   Message( "Generating page...")
   cText := "DJ.lib A Clipper library for HP DeskJet Printers (c) Andrew Murray 1995"
   nRow  := 20
   dj_SetFont( "UNIVERS__14" )
   dj_Say( nRow, nHCentre, cText, "C" )

   cText := "Ratsoft    Tel: +49 89 2711262    Fax: +49 89 2718204    CIS: 100016,3406"
   nRow  += dj_CharHt()                       // Increment row by height of current font
   dj_SetFont( "UNIVERS__10" )
   dj_Say( nRow, nHCentre, cText, "C" )

   // Draw a horizontal line between the margins
   nRow  := 20
   dj_LineH( nTMargin+nRow, nLMargin, nRMargin-nLMargin)

   // Debugging ...
   dj_SetFont( "UNIVERS__6" )
   nRow := 13
   nCol := 11
   cTmp := "Metric"
   dj_Say( nRow, nCol, cTmp, "L" )


   //
   // Testing.....
   //
   cFont :=  "CGTIMES__6"
   dj_SetFont( cFont )
   nRow := 40
   nCol := 11
   cTmp := "Andrew Murray, Elisabethstrae 31, 80796 Mnchen"
   dj_Say( nRow, nCol, cTmp, "L" )

   // Draw a line under the address
   // Note how the font is passed to this function ----------
   // which in this case is not strictly necessary
   dj_Line( nRow, 11, dj_StrLen(cTmp), DJ_LINE_HORIZONTAL, cFont)

   // And now the customer number in a box under this line
   cTmp := "1234567890"
   // I make the box height ( nHt ) font dependant...
   nH   := dj_CharHt()   // Get actual font height of current font
   nCol += 1
   dj_Say( nRow+nH, nCol, cTmp )

   // Now box the customer number, passing the font ----
   nRow := 40
   dj_Box( nRow, 11, nRow+nH, 11+dj_StrLen( cTmp )+2, cFont )

   //
   // Finished testing .....
   //


   //
   // And now send contents of DeskJet page buffer
   //
   Message( "Sending page to printer...")
   dj_Print()
   dj_Eject()
   dj_Reset()


   // Finished
   pr_Set( FALSE )
   Message( "Finished!")
   Message( "")

return( NIL )


//
// Print out heading lines
//
static function Heading( nRow, cText )

   local nCol := NIL

   dj_SetFont( "CGTIMES__12_B" )
   #ifdef METRIC
      nCol := 15
   #else
      nCol := 0.59
   #endif

   dj_Say( nRow, nCol, cText, "L" )
return( NIL )


//
// Send a messge to screen
//
static function Message( cText )

   local cOldDev := set(_SET_DEVICE, "SCREEN") ,;
         lOldSet := pr_Set( FALSE )

   Qout( cText )
   pr_Set( lOldSet )
   set(_SET_DEVICE, cOldDev )

return( NIL )


//
// Miscellaneous functions for DJ lib
//


//
// Extract a token from another based on delimiter
//
// This is a stub function for the one in FunckyII referenced
// by DJ.lib
//
// Note that this function only handles the simple case of
// a single delimiter since this is all that DJ.lib requires
//
// For example:
//
//   strextract( "CGTIMES__12", "_", 3 ) returns "12"
//   strextract( "CGTIMES__12", "_", 2 ) returns ""
//   strextract( "__12", "_", 3 )        returns "12"
//
function strextract( cStr, cDelim, nOccur )

   local cRetval     := ""    ,;     // Return value, defaults to ""
         nCnt        := 0     ,;     // Counter for tokens extracted
         nPos        := NIL   ,;
         lContinue   := .T.   ,;
         cTok        := NIL

   // Keep extracting and truncating until token found or string exhausted
   while ( lContinue .and.  !empty(cStr) )

      nCnt++
      nPos := at( cDelim, cStr )

      if ( nPos > 0 )
         if (nPos == 1)
            cTok := ""
         else
            cTok := substr( cStr, 1, nPos-1 )
         endif

         if ( nCnt == nOccur )
            cRetval   := cTok
            lContinue := .F.
         else
            cStr := substr( cStr, nPos + 1 )
         endif

      else

         if ( nCnt == nOccur )
            cRetval := cStr
         endif

         lContinue := .F.

      endif

   enddo

return( cRetval )



